package net.sourceforge.openutils.mgnlgroovy.pages;

import groovy.lang.GroovyShell;
import groovy.lang.Script;
import info.magnolia.cms.beans.runtime.Document;
import info.magnolia.cms.util.AlertUtil;
import info.magnolia.module.admininterface.TemplatedMVCHandler;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringUtils;
import org.codehaus.groovy.control.CompilationFailedException;
import org.codehaus.groovy.runtime.InvokerHelper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * A basic Groovy shell to be embedded in Magnolia.
 * 
 * @author fgrilli
 * @version $Id: GroovyShellPage.java 5074 2008-10-21 13:16:36Z federico.grilli $
 */
public class GroovyShellPage extends TemplatedMVCHandler
{
    private static final String LINE_SEP = System.getProperty("line.separator");

    private static final String groovyVersion = InvokerHelper.getVersion();

    private static final long serialVersionUID = 222L;

    private static final Logger log = LoggerFactory.getLogger(GroovyShellPage.class);

    private String groovyScript;

    private Document mgnlFileImport;

    public GroovyShellPage(String name, HttpServletRequest request, HttpServletResponse response)
    {
        super(name, request, response);

    }

    public String executeGroovyScript()
    {
        if (StringUtils.isBlank(groovyScript))
        {
            String msg = "No script to execute";
            log.warn(msg);
            AlertUtil.setMessage(msg);
            return this.show();
        }
        if (log.isDebugEnabled())
        {
            log.debug("executing script {}", groovyScript);
        }

        Object retVal;

        try
        {
            GroovyShell shell = new GroovyShell();
            Script script = shell.parse(getGroovyScript());
            retVal = script.run();
            AlertUtil.setMessage("Script executed successfully. Return value is " + retVal);
        }
        catch (CompilationFailedException e)
        {
            log.error("An error occurred while parsing the script. Error message is {}", new Object[]
            { e.getMessage() });
            AlertUtil.setMessage("An error occurred while parsing the script. Error message is " + e.getMessage());
        }
        catch (Exception e)
        {
            log.error("An error occurred while executing the script. Error message is {}", new Object[]
            { e.getMessage() });
            AlertUtil.setMessage("An error occurred while executing the script. Error message is " + e.getMessage());
        }

        return this.show();
    }

    public String getGroovyScript()
    {
        return groovyScript;
    }

    public String getGroovyVersion()
    {
        return groovyVersion;
    }

    public Document getMgnlFileImport()
    {
        return mgnlFileImport;
    }

    public String loadGroovyScript()
    {
        if (mgnlFileImport == null)
        {
            String msg = "Please, select a file";
            log.warn(msg);
            AlertUtil.setMessage(msg);
            return this.show();
        }
        if (!mgnlFileImport.getExtension().equalsIgnoreCase("groovy"))
        {
            String msg = mgnlFileImport.getFileNameWithExtension() + " doesn't seem to be a valid groovy file";
            log.warn(msg);
            AlertUtil.setMessage(msg);
            return this.show();
        }
        groovyScript = readFile(mgnlFileImport.getFile());

        String msg = mgnlFileImport.getFileNameWithExtension() + " loaded";
        log.info(msg);
        AlertUtil.setMessage(msg);
        return this.show();

    }

    private String readFile(File file)
    {
        StringBuilder contents = new StringBuilder();

        try
        {
            // use buffering, reading one line at a time
            // FileReader always assumes default encoding is OK!
            BufferedReader input = new BufferedReader(new FileReader(file));
            try
            {
                String line = null; // not declared within while loop
                /*
                * readLine is a bit quirky :
                * it returns the content of a line MINUS the newline.
                * it returns null only for the END of the stream.
                * it returns an empty String if two newlines appear in a row.
                */
                while ((line = input.readLine()) != null)
                {
                    contents.append(line);
                    contents.append(LINE_SEP);
                }
            }
            finally
            {
                IOUtils.closeQuietly(input);
            }
        }
        catch (IOException ex)
        {
            log.error(ex.getMessage());
        }
        return contents.toString();
    }

    public void setGroovyScript(String groovyScript)
    {
        this.groovyScript = groovyScript;
    }

    public void setMgnlFileImport(Document mgnlFileImport)
    {
        this.mgnlFileImport = mgnlFileImport;
    }
}